/* Helper module for testing Win32 implementations.
  Example of usage:

    ok(1 == 1, "Successful test");

    START_TODO(get_w32_impl() == W32_WINE,
        "This feature is not implemented in Wine");
    ok(1 == 2, "Failing TODO test");
    END_TODO;

    ok(1 == 2, "Failing test");

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wt_helper.h"

#define TRUE 1
#define FALSE 0

/* counter of TODO sections nesting */
int todo_level = 0;

/* indicator of the current active level of TODO sections */
int active_todo = 0;

/* current TODO description. Is valid any time. Equals "" if not available */
char *todo_descr = "";

/*
 * Returns Win32 API implementation the program is running on
 */
w32_impl get_w32_impl()
{
    /* TODO!!! define current platform at the run-time.
       This means the whole module will be dependent on windows.h */
    return W32_MS;
}

/*
 * Internally used function for processing test failure.
 * Currently prints message and exists with error code 1.
 *   - msg - message to print
 *   - file - test application source code file name
 *   - line - test application source code file line number
 */
void process_failure(char *msg, char *file, int line)
{
    fprintf(stderr, "%s : file %s, line %i\n", msg, file, line);
    exit(1);
}

/*
 * Starts TODO section. Not thread-safe.
 * params:
 *   - condition - TODO condition;
 *   - descr - valid string with TODO description
 */
void _start_todo(int condition, char *descr, char *file, int line)
{
    todo_level++;
    if (condition)
    {
        if (active_todo != 0)
        {
            process_failure("Active TODO sections can't be nested",
                            file, line);
        }
        if (todo_level == 0)
        {
            process_failure("Incorrect nesting of TODO sections",
                            file, line);
        }
        
        active_todo = todo_level;
        todo_descr = malloc(strlen(descr) + 1);
        strcpy(todo_descr, descr);
    }
}

/*
 * Ends TODO section.
 */
void _end_todo(char *file, int line)
{
    todo_level--;
    if (todo_level < 0)
    {
        process_failure(
            "Incorrect processing of TODO nestings - negative TODO level",
            file, line);
    }
    
    if ((active_todo != 0) && (todo_level == active_todo))
    {
        active_todo = 0;
        free(todo_descr);
        todo_descr = "";
    }
}

/*
 * Checks condition.
 * Parameters:
 *   - condition - condition to check;
 *   - msg test description;
 *   - file - test application source code file name of the check
 *   - line - test application source code file line number of the check
 */
void _ok(int condition, char *msg, char *file, int line)
{
    if (!condition && (active_todo == 0))
    {
        process_failure(msg, file, line);
    }

    if (condition && (active_todo != 0))
    {
        char *todo_test_succeded =
            "CONGRATULATIONS!!! Your TODO test succeded. "
            "Move it out of the TODO section";
        char *s = malloc(
            strlen(strlen(todo_descr) + todo_test_succeded) + strlen(msg) + 15);
        sprintf(s, "%s\n%s : #TODO %s", todo_test_succeded, msg, todo_descr );
        process_failure(s, file, line);
        free(s);
    }
}

